# -*- coding: utf-8 -*-
import threading
from helpers import *
import time
import requests
import StringIO

import xbmcaddon
__addon__ = xbmcaddon.Addon()
__scriptname__ = __addon__.getAddonInfo('name')

__url__ = None
__headers__ = {'Content-Type': 'application/octet-stream'}

def take_snapshot(cap):
  # Only proceed if the capture has succeeded!
  while True:
    cap.waitForCaptureStateChangeEvent(5)
    if cap.getCaptureState() == xbmc.CAPTURE_STATE_DONE:
      break
  return cap.getImage()

def led_detect(dev_name):
  url = 'http://%s/bmp' % dev_name
  res = requests.post(url=url, data=get_start(), headers=__headers__)
  if res.status_code == requests.codes.ok and res.text == 'ok !':
    notify(__scriptname__, '%s found %s' % (dev_name, res.text,))
    global __url__
    __url__ = url
  else:
    # Device is not found
    notify(__scriptname__, '%s is not found' % (dev_name,))

def led_set(data):
    if __url__:
      # Send data to leds
      try:
        res = requests.post(url=__url__, data=data, headers=__headers__, timeout=1.0)
      except:
        log('Error post %s' % __url__)
        pass

def img_proc(w, h, rate, dbg_fcnt, alpha, fname, r, g, b, gamma_corr, sat, dev_name, full, stop):
  thr = threading.current_thread()
  cap = xbmc.RenderCapture()
  cap.capture(w, h, (xbmc.CAPTURE_FLAG_CONTINUOUS | xbmc.CAPTURE_FLAG_IMMEDIATELY))
  start_time = None
  mtx = get_rgb2rgb(sat)
  cnt = 0

  led_detect(dev_name)

  while not stop.is_set():
    if xbmc.Player().isPlayingVideo():
      corr_start = time.time()
      pix = take_snapshot(cap)

      output = StringIO.StringIO()
      img = Image.new( 'RGB', (cap.getWidth(), cap.getHeight()), 'black')
      pixels = img.load()

      for x, y, z, w, h in extract_pixes(pix, cap.getWidth(), cap.getHeight(), alpha, full):
        # rgb2rgb
        x1 = clamp(0, x * mtx['rr'] + y * mtx['rg'] + z * mtx['rb'], 255)
        y1 = clamp(0, x * mtx['gr'] + y * mtx['gg'] + z * mtx['gb'], 255)
        z1 = clamp(0, x * mtx['br'] + y * mtx['bg'] + z * mtx['bb'], 255)
        pixels[w, h] = (gamma(x1, r, gamma_corr), gamma(y1, g, gamma_corr), gamma(z1, b, gamma_corr))

      #img = led_data.transpose(Image.FLIP_TOP_BOTTOM)
      img.save(output, format='bmp')

      led_set(output.getvalue())
      #save_bmp(output.getvalue())
      output.close()

      if not (cnt % dbg_fcnt) and fname:
        if start_time:
          log ('%s %d Fps: %f ' % (thr.name, cnt, (1/((time.time() - start_time)/dbg_fcnt))))
          save_png(bytes(pix), (cap.getWidth(), cap.getHeight()), cap.getImageFormat(), fname)
        start_time = time.time()

      cnt += 1
      xbmc.sleep(max(1, int((rate-(time.time()-corr_start)) * 1000)))
    else:
      if xbmc.Player().isPlaying() and __url__:
        try:
          res = requests.post(url=__url__, data=get_start(), headers=__headers__, timeout=1.0)
        except:
          log('Error post %s' % __url__)
          pass
      xbmc.sleep(1000)

  stop.clear()
